#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" The package is used to define detailed customer models.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM.IEC61968.Common import Agreement
from CIM.IEC61968.Common import Document
from CIM.IEC61970.Core import Curve
from CIM.IEC61970.Domain import AbsoluteDate
from CIM.IEC61970.Domain import Money
from CIM.IEC61970.Domain import Voltage



from enthought.traits.api import Instance, List, Property, Enum, Str, Date, Float, Bool
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------

# Kind of customer billing.
CustomerBillingKind = Enum("separateEssUdc", "other", "consolidatedEss", "consolidatedUdc", desc="Kind of customer billing.")

#------------------------------------------------------------------------------
#  "ExternalCustomerAgreement" class:
#------------------------------------------------------------------------------

class ExternalCustomerAgreement(Agreement):
    """ A type of customer agreement involving an external agency. For example, a customer may form a contracts with an Energy Service Supplier if Direct Access is permitted.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "ExternalCustomerAgreement" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "signDate",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "validityInterval",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfCustomers.ExternalCustomerAgreement",
        title="ExternalCustomerAgreement",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ExternalCustomerAgreement" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "StandardIndustryCode" class:
#------------------------------------------------------------------------------

class StandardIndustryCode(Document):
    """ The Standard Industrial Classification (SIC) are the codes that identify the type of products/service an industry is involved in, and used for statutory reporting purposes. For example, in the USA these codes are located by the federal government, and then published in a book entitled 'The Standard Industrial Classification Manual'. The codes are arranged in a hierarchical structure. Note that Residential Service Agreements are not classified according to the SIC codes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    CustomerAgreements = List(Instance("CIM.IEC61968.Customers.CustomerAgreement"))

    # Standard alphanumeric code assigned to a particular product/service within an industry.
    code = Str(desc="Standard alphanumeric code assigned to a particular product/service within an industry.")

    #--------------------------------------------------------------------------
    #  Begin "StandardIndustryCode" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "code",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CustomerAgreements",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfCustomers.StandardIndustryCode",
        title="StandardIndustryCode",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "StandardIndustryCode" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CustomerBillingInfo" class:
#------------------------------------------------------------------------------

class CustomerBillingInfo(Document):
    """ The creation of the monthly customer billing statements is the method employed to notify Customers of charges, adjustments and credits applied to their account for Services and Products. The actuall billing occurs through an ErpInvoice. The CustomerBillingInfo includes information from the payment, collection, meter reading, installed meter, service, site, customer, customer account, customer agreement, services and pricing subject areas. Each component price shows up as a separate line item on the ErpInvoice. The Customer Billing Statement may include collection and account messages, marketing/civic event messages and bill inserts. One Customer Billing Statement is produced for all Agreements under a CustomerAccount per billing cycle date defined in 'CustomerAccount.billingCycle'. The history of CustomerBillingInfo, Invoices and Payments is to be maintained in associated ActivityRecords.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ErpInvoiceLineItems = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpInvoiceLineItem"))

    CustomerAccount = Instance("CIM.IEC61968.Customers.CustomerAccount",
        transient=True,
        opposite="CustomerBillingInfos",
        editor=InstanceEditor(name="_customeraccounts"))

    def _get_customeraccounts(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Customers.CustomerAccount" ]
        else:
            return []

    _customeraccounts = Property(fget=_get_customeraccounts)

    # Kind of bill customer receives.
    kind = CustomerBillingKind(desc="Kind of bill customer receives.")

    # Type of payment plan.
    pymtPlanType = Str(desc="Type of payment plan.")

    # Business date designated for the billing run which produced this CustomerBillingInfo.
    billingDate = AbsoluteDate(desc="Business date designated for the billing run which produced this CustomerBillingInfo.")

    # Monthly amortized amount due during each billing cycle for the CustomerAccount balance for which the Payment Plan is set-up.
    pymtPlanAmt = Money(desc="Monthly amortized amount due during each billing cycle for the CustomerAccount balance for which the Payment Plan is set-up.")

    # Outstanding balance on the CustomerAccount as of the statement date.
    outBalance = Money(desc="Outstanding balance on the CustomerAccount as of the statement date.")

    # Amount of the last payment received from the customer. It is retained in the Customer Billing system, although the details of each payment are tracked in the ERP system.
    lastPaymentAmt = Money(desc="Amount of the last payment received from the customer. It is retained in the Customer Billing system, although the details of each payment are tracked in the ERP system.")

    # Date of the last payment received from the customer. It is retained in the Customer Billing system, although the details of each payment are tracked in the ERP system.
    lastPaymentDate = AbsoluteDate(desc="Date of the last payment received from the customer. It is retained in the Customer Billing system, although the details of each payment are tracked in the ERP system.")

    # Calculated date upon which a customer billing amount is due, used in the invoicing process to determine when a Customer's Payment is delinquent. It takes into consideration the regulatory criteria and the Customer's requested due date. In the absence of a Customer requested due date, the due date is typically calculated from the regulated number of days and the 'billingDate'.
    dueDate = AbsoluteDate(desc="Calculated date upon which a customer billing amount is due, used in the invoicing process to determine when a Customer's Payment is delinquent. It takes into consideration the regulatory criteria and the Customer's requested due date. In the absence of a Customer requested due date, the due date is typically calculated from the regulated number of days and the 'billingDate'.")

    #--------------------------------------------------------------------------
    #  Begin "CustomerBillingInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "kind", "pymtPlanType", "billingDate", "pymtPlanAmt", "outBalance", "lastPaymentAmt", "lastPaymentDate", "dueDate",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "ErpInvoiceLineItems", "CustomerAccount",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfCustomers.CustomerBillingInfo",
        title="CustomerBillingInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CustomerBillingInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OutageHistory" class:
#------------------------------------------------------------------------------

class OutageHistory(Document):
    """ A document collecting OutageReports, that allows utilities to examine the number of outages suffered by a customer. Also provides data to calculate the total supply interruption to any customer over a given period.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # OutageReports per customer for which this OutageHistory is created.
    OutageReports = List(Instance("CIM.IEC61968.Informative.InfOperations.OutageReport"),
        desc="OutageReports per customer for which this OutageHistory is created.")

    #--------------------------------------------------------------------------
    #  Begin "OutageHistory" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "OutageReports",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfCustomers.OutageHistory",
        title="OutageHistory",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OutageHistory" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "WorkBillingInfo" class:
#------------------------------------------------------------------------------

class WorkBillingInfo(Document):
    """ Billing information for work performed for the customer. The history of Work Billing Info, Invoices, and Payments is to be maintained in associated ActivityRecords.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ErpLineItems = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpInvoiceLineItem"))

    CustomerAccount = Instance("CIM.IEC61968.Customers.CustomerAccount",
        transient=True,
        opposite="WorkBillingInfos",
        editor=InstanceEditor(name="_customeraccounts"))

    def _get_customeraccounts(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Customers.CustomerAccount" ]
        else:
            return []

    _customeraccounts = Property(fget=_get_customeraccounts)

    Works = List(Instance("CIM.IEC61968.Work.Work"))

    # Date and time by which payment for bill is expected from client.
    dueDateTime = Date(desc="Date and time by which payment for bill is expected from client.")

    # Date payment was received from client.
    receivedDateTime = Date(desc="Date payment was received from client.")

    # Amount of price on deposit.
    deposit = Money(desc="Amount of price on deposit.")

    # Amount of bill.
    workPrice = Money(desc="Amount of bill.")

    # Discount from standard price.
    discount = Float(desc="Discount from standard price.")

    # Estimated cost for work.
    costEstimate = Money(desc="Estimated cost for work.")

    # Date and time bill was issued to client.
    issueDateTime = Date(desc="Date and time bill was issued to client.")

    #--------------------------------------------------------------------------
    #  Begin "WorkBillingInfo" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "dueDateTime", "receivedDateTime", "deposit", "workPrice", "discount", "costEstimate", "issueDateTime",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "ErpLineItems", "CustomerAccount", "Works",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfCustomers.WorkBillingInfo",
        title="WorkBillingInfo",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "WorkBillingInfo" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "PowerQualityPricing" class:
#------------------------------------------------------------------------------

class PowerQualityPricing(Document):
    """ Pricing can be based on power quality.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    PricingStructure = Instance("CIM.IEC61968.Customers.PricingStructure",
        transient=True,
        opposite="PowerQualityPricings",
        editor=InstanceEditor(name="_pricingstructures"))

    def _get_pricingstructures(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Customers.PricingStructure" ]
        else:
            return []

    _pricingstructures = Property(fget=_get_pricingstructures)

    ServiceDeliveryPoints = List(Instance("CIM.IEC61968.Metering.ServiceDeliveryPoint"))

    # Emergency low voltage limit.
    emergencyLowVoltLimit = Voltage(desc="Emergency low voltage limit.")

    # Emergency high voltage limit.
    emergencyHighVoltLimit = Voltage(desc="Emergency high voltage limit.")

    # Voltage limit violation cost (Cost per unit Voltage).
    voltLimitViolCost = Float(desc="Voltage limit violation cost (Cost per unit Voltage).")

    # Value of uninterrupted service (Cost per active power).
    valueUninterruptedServiceP = Float(desc="Value of uninterrupted service (Cost per active power).")

    # Normal low voltage limit.
    normalLowVoltLimit = Voltage(desc="Normal low voltage limit.")

    # Voltage imbalance violation cost (Cost per unit Voltage).
    voltImbalanceViolCost = Float(desc="Voltage imbalance violation cost (Cost per unit Voltage).")

    # Normal high voltage limit.
    normalHighVoltLimit = Voltage(desc="Normal high voltage limit.")

    # Threshold minimum power factor for this PricingStructure, specified in instances where a special charge is levied if the actual power factor for a Service falls below the value specified here.
    powerFactorMin = Float(desc="Threshold minimum power factor for this PricingStructure, specified in instances where a special charge is levied if the actual power factor for a Service falls below the value specified here.")

    # Value of uninterrupted service (Cost per energy).
    valueUninterruptedServiceEnergy = Float(desc="Value of uninterrupted service (Cost per energy).")

    #--------------------------------------------------------------------------
    #  Begin "PowerQualityPricing" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "emergencyLowVoltLimit", "emergencyHighVoltLimit", "voltLimitViolCost", "valueUninterruptedServiceP", "normalLowVoltLimit", "voltImbalanceViolCost", "normalHighVoltLimit", "powerFactorMin", "valueUninterruptedServiceEnergy",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "PricingStructure", "ServiceDeliveryPoints",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfCustomers.PowerQualityPricing",
        title="PowerQualityPricing",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "PowerQualityPricing" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ServiceGuarantee" class:
#------------------------------------------------------------------------------

class ServiceGuarantee(Document):
    """ A service guarantee, often imposed by a regulator, defines conditions that, if not satisfied, will result in the utility making a monetary payment to the customer. Note that guarantee's identifier is in the 'name' attribute and the status of the guarantee is in the 'Status.status' attribute. Example service requirements include: 1) If power is not restored within 24 hours, customers can claim $50 for residential customers or $100 for commercial and industrial customers. In addition for each extra period of 12 hours the customer's supply has not been activated, the customer can claim $25. 2) If a customer has a question about their electricity bill, the utility will investigate and respond to the inquiry within 15 working days. If utility fails to meet its guarantee, utility will automatically pay the customer $50.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Period in which this service guantee applies.
    applicationPeriod = Instance("CIM.IEC61968.Common.DateTimeInterval",
        desc="Period in which this service guantee applies.",
        transient=True,
        editor=InstanceEditor(name="_datetimeintervals"))

    def _get_datetimeintervals(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.DateTimeInterval" ]
        else:
            return []

    _datetimeintervals = Property(fget=_get_datetimeintervals)

    # True if utility must autmatically pay the specified amount whenever the condition is not satisified, otherwise customer must make a claim to receive payment.
    automaticPay = Bool(desc="True if utility must autmatically pay the specified amount whenever the condition is not satisified, otherwise customer must make a claim to receive payment.")

    # Explanation of the requirement and conditions for satisfying it.
    serviceRequirement = Str(desc="Explanation of the requirement and conditions for satisfying it.")

    # Amount to be paid by the service provider to the customer for each violation of the 'serviceRequirement'.
    payAmount = Money(desc="Amount to be paid by the service provider to the customer for each violation of the 'serviceRequirement'.")

    #--------------------------------------------------------------------------
    #  Begin "ServiceGuarantee" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "automaticPay", "serviceRequirement", "payAmount",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "applicationPeriod",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfCustomers.ServiceGuarantee",
        title="ServiceGuarantee",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ServiceGuarantee" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SubscribePowerCurve" class:
#------------------------------------------------------------------------------

class SubscribePowerCurve(Curve):
    """ Price curve for specifying the cost of energy (X) at points in time (y1) according to a prcing structure, which is based on a tariff.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    PricingStructure = Instance("CIM.IEC61968.Customers.PricingStructure",
        transient=True,
        opposite="SubscribePowerCurve",
        editor=InstanceEditor(name="_pricingstructures"))

    def _get_pricingstructures(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Customers.PricingStructure" ]
        else:
            return []

    _pricingstructures = Property(fget=_get_pricingstructures)

    #--------------------------------------------------------------------------
    #  Begin "SubscribePowerCurve" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "y3Multiplier", "y2Multiplier", "xMultiplier", "y2Unit", "curveStyle", "y1Unit", "y1Multiplier", "y3Unit", "xUnit",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "CurveDatas", "PricingStructure",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfCustomers.SubscribePowerCurve",
        title="SubscribePowerCurve",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SubscribePowerCurve" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" Contains only diagrams to be discussed with WG13, for consolidating T&amp;D models.Contains only diagrams to be discussed with WG13, for consolidating T&amp;D models.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CDPSM.IEC61970.Core import ConductingEquipment
from CDPSM.IEC61970.Wires import ACLineSegment
from CDPSM.IEC61970.Core import IdentifiedObject
from CDPSM.IEC61970.Wires import RatioTapChanger
from CDPSM.IEC61970.Core import Equipment
from CDPSM import Element
from CDPSM.IEC61970.Domain import Resistance
from CDPSM.IEC61970.Domain import Reactance
from CDPSM.IEC61970.Domain import Conductance
from CDPSM.IEC61970.Domain import Susceptance
from CDPSM.IEC61970.Core import PhaseCode
from CDPSM.IEC61970.Domain import Voltage



from enthought.traits.api import Instance, List, Property, Bool, Float, Str, Int
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#  "DistributionTransformerWinding" class:
#------------------------------------------------------------------------------

class DistributionTransformerWinding(ConductingEquipment):
    """ Conducting connection point of a distribution / unbalanced transformer winding instance. This class differs from Wires::TransformerWinding as follows: - the eight Pi model attributes are moved into separate class, that can be optionally referred to from several winding instances. - the three grounding attributes can differ per winding instance, even for windings that use the same TransformerInfo, so they are kept on DistributionTransformerWinding. 'sequenceNumber' attribute replaces 'windingType' attribute.  All the other attributes come from the WindingInfo (and its relationships). TransformerInfo is associated to the DistributionTransformer as referenceable data, so it can be defined once and referred to from instances, instead of being specified with each instance.Conducting connection point of a distribution / unbalanced transformer winding instance. This class differs from Wires::TransformerWinding as follows: - the eight Pi model attributes are moved into separate class, that can be optionally referred to from several winding instances. - the three grounding attributes can differ per winding instance, even for windings that use the same TransformerInfo, so they are kept on DistributionTransformerWinding. 'sequenceNumber' attribute replaces 'windingType' attribute.  All the other attributes come from the WindingInfo (and its relationships). TransformerInfo is associated to the DistributionTransformer as referenceable data, so it can be defined once and referred to from instances, instead of being specified with each instance.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Data for this winding.Data for this winding.
    WindingInfo = Instance("CDPSM.IEC61968.AssetModels.WindingInfo",
        desc="Data for this winding.Data for this winding.",
        transient=True,
        opposite="Windings",
        editor=InstanceEditor(name="_windinginfos"))

    def _get_windinginfos(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.AssetModels.WindingInfo" ]
        else:
            return []

    _windinginfos = Property(fget=_get_windinginfos)

    # Transformer this winding belongs to.Transformer this winding belongs to.
    Transformer = Instance("CDPSM.IEC61968.WiresExt.DistributionTransformer",
        desc="Transformer this winding belongs to.Transformer this winding belongs to.",
        transient=True,
        opposite="Windings",
        editor=InstanceEditor(name="_distributiontransformers"))

    def _get_distributiontransformers(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.WiresExt.DistributionTransformer" ]
        else:
            return []

    _distributiontransformers = Property(fget=_get_distributiontransformers)

    # Ratio tap changer associated with this winding.Ratio tap changer associated with this winding.
    RatioTapChanger = Instance("CDPSM.IEC61970.Wires.RatioTapChanger",
        desc="Ratio tap changer associated with this winding.Ratio tap changer associated with this winding.",
        transient=True,
        opposite="Winding",
        editor=InstanceEditor(name="_ratiotapchangers"))

    def _get_ratiotapchangers(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61970.Wires.RatioTapChanger" ]
        else:
            return []

    _ratiotapchangers = Property(fget=_get_ratiotapchangers)

    # (accurate for 2- or 3-winding transformers only) Pi-model impedances of this winding.(accurate for 2- or 3-winding transformers only) Pi-model impedances of this winding.
    PiImpedance = Instance("CDPSM.IEC61968.WiresExt.WindingPiImpedance",
        desc="(accurate for 2- or 3-winding transformers only) Pi-model impedances of this winding.(accurate for 2- or 3-winding transformers only) Pi-model impedances of this winding.",
        transient=True,
        opposite="Windings",
        editor=InstanceEditor(name="_windingpiimpedances"))

    def _get_windingpiimpedances(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.WiresExt.WindingPiImpedance" ]
        else:
            return []

    _windingpiimpedances = Property(fget=_get_windingpiimpedances)

    # (for Yn and Zn connections) Resistance part of neutral impedance where 'grounded' is true.(for Yn and Zn connections) Resistance part of neutral impedance where 'grounded' is true.
    rground = Resistance(desc="(for Yn and Zn connections) Resistance part of neutral impedance where 'grounded' is true.(for Yn and Zn connections) Resistance part of neutral impedance where 'grounded' is true.")

    # (for Yn and Zn connections) Reactive part of neutral impedance where 'grounded' is true.(for Yn and Zn connections) Reactive part of neutral impedance where 'grounded' is true.
    xground = Reactance(desc="(for Yn and Zn connections) Reactive part of neutral impedance where 'grounded' is true.(for Yn and Zn connections) Reactive part of neutral impedance where 'grounded' is true.")

    # (for Yn and Zn connections) True if the neutral is solidly grounded.(for Yn and Zn connections) True if the neutral is solidly grounded.
    grounded = Bool(desc="(for Yn and Zn connections) True if the neutral is solidly grounded.(for Yn and Zn connections) True if the neutral is solidly grounded.")

    #--------------------------------------------------------------------------
    #  Begin "DistributionTransformerWinding" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "normaIlyInService", "phases", "rground", "xground", "grounded",
                label="Attributes"),
            VGroup("Model", "GeoLocation", "PSRType", "EquipmentContainer", "Terminals", "BaseVoltage", "WindingInfo", "Transformer", "RatioTapChanger", "PiImpedance",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.DistributionTransformerWinding",
        title="DistributionTransformerWinding",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DistributionTransformerWinding" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DistributionLineSegment" class:
#------------------------------------------------------------------------------

class DistributionLineSegment(ACLineSegment):
    """ Extends ACLineSegment with references to a library of standard types from which electrical parameters can be calculated, as follows: - calculate electrical parameters from asset data, using associated ConductorInfo, with values then multiplied by Conductor.length to produce a matrix model. - calculate unbalanced electrical parameters from associated PerLengthPhaseImpedance, then multiplied by Conductor.length to produce a matrix model. - calculate transposed electrical parameters from associated PerLengthSequenceImpedance, then multiplied by Conductor.length to produce a sequence model. For symmetrical, transposed 3ph lines, it is sufficient to use inherited ACLineSegment attributes, which describe sequence impedances and admittances for the entire length of the segment.  Known issue: Attributes expressing impedances and admittances in PerLengthSequenceImpedance and PhaseImpedanceData use Resistance, etc., which describe pre-calculated, full length of segment, while we should have a longitudinal unit, per length. Taking 'r' as example, its 'unit'=Ohm, but the value is effectively in Ohm/m, so the value needs to be multiplied by Conductor.length. This is against the whole idea of unit data types and is semantically wrong, but base CIM does not have the required data types at this moment. Until the revision of unit modelling in CIM, applications need to deduce and locally handle appending '/m' for units and ensure they multiply the values by Conductor.length.At least one of the Associations must exist.Extends ACLineSegment with references to a library of standard types from which electrical parameters can be calculated, as follows: - calculate electrical parameters from asset data, using associated ConductorInfo, with values then multiplied by Conductor.length to produce a matrix model. - calculate unbalanced electrical parameters from associated PerLengthPhaseImpedance, then multiplied by Conductor.length to produce a matrix model. - calculate transposed electrical parameters from associated PerLengthSequenceImpedance, then multiplied by Conductor.length to produce a sequence model. For symmetrical, transposed 3ph lines, it is sufficient to use inherited ACLineSegment attributes, which describe sequence impedances and admittances for the entire length of the segment.  Known issue: Attributes expressing impedances and admittances in PerLengthSequenceImpedance and PhaseImpedanceData use Resistance, etc., which describe pre-calculated, full length of segment, while we should have a longitudinal unit, per length. Taking 'r' as example, its 'unit'=Ohm, but the value is effectively in Ohm/m, so the value needs to be multiplied by Conductor.length. This is against the whole idea of unit data types and is semantically wrong, but base CIM does not have the required data types at this moment. Until the revision of unit modelling in CIM, applications need to deduce and locally handle appending '/m' for units and ensure they multiply the values by Conductor.length.At least one of the Associations must exist.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Conductor data of this conductor segment.Conductor data of this conductor segment.
    ConductorInfo = Instance("CDPSM.IEC61968.AssetModels.ConductorInfo",
        desc="Conductor data of this conductor segment.Conductor data of this conductor segment.",
        transient=True,
        opposite="ConductorSegments",
        editor=InstanceEditor(name="_conductorinfos"))

    def _get_conductorinfos(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.AssetModels.ConductorInfo" ]
        else:
            return []

    _conductorinfos = Property(fget=_get_conductorinfos)

    # Sequence impedance of this conductor segment; used for balanced model.Sequence impedance of this conductor segment; used for balanced model.
    SequenceImpedance = Instance("CDPSM.IEC61968.WiresExt.PerLengthSequenceImpedance",
        desc="Sequence impedance of this conductor segment; used for balanced model.Sequence impedance of this conductor segment; used for balanced model.",
        transient=True,
        opposite="ConductorSegments",
        editor=InstanceEditor(name="_perlengthsequenceimpedances"))

    def _get_perlengthsequenceimpedances(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.WiresExt.PerLengthSequenceImpedance" ]
        else:
            return []

    _perlengthsequenceimpedances = Property(fget=_get_perlengthsequenceimpedances)

    # Phase impedance of this conductor segment; used for unbalanced model.Phase impedance of this conductor segment; used for unbalanced model.
    PhaseImpedance = Instance("CDPSM.IEC61968.WiresExt.PerLengthPhaseImpedance",
        desc="Phase impedance of this conductor segment; used for unbalanced model.Phase impedance of this conductor segment; used for unbalanced model.",
        transient=True,
        opposite="ConductorSegments",
        editor=InstanceEditor(name="_perlengthphaseimpedances"))

    def _get_perlengthphaseimpedances(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.WiresExt.PerLengthPhaseImpedance" ]
        else:
            return []

    _perlengthphaseimpedances = Property(fget=_get_perlengthphaseimpedances)

    #--------------------------------------------------------------------------
    #  Begin "DistributionLineSegment" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "normaIlyInService", "phases", "length", "r", "x0", "bch", "x", "b0ch", "r0",
                label="Attributes", columns=1),
            VGroup("Model", "GeoLocation", "PSRType", "EquipmentContainer", "Terminals", "BaseVoltage", "ConductorInfo", "SequenceImpedance", "PhaseImpedance",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.DistributionLineSegment",
        title="DistributionLineSegment",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DistributionLineSegment" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "WindingPiImpedance" class:
#------------------------------------------------------------------------------

class WindingPiImpedance(IdentifiedObject):
    """ Transformer Pi-model impedance that accurately reflects impedance for transformers with 2 or 3 windings. For transformers with 4 or more windings, you must use TransformerInfo.Transformer Pi-model impedance that accurately reflects impedance for transformers with 2 or 3 windings. For transformers with 4 or more windings, you must use TransformerInfo.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All windings having this Pi impedance.All windings having this Pi impedance.
    Windings = List(Instance("CDPSM.IEC61968.WiresExt.DistributionTransformerWinding"),
        desc="All windings having this Pi impedance.All windings having this Pi impedance.")

    # Positive sequence series reactance of the winding.  For a two winding transformer, the full reactance of the transformer should be entered on the primary (high voltage) winding.Positive sequence series reactance of the winding.  For a two winding transformer, the full reactance of the transformer should be entered on the primary (high voltage) winding.
    x = Reactance(desc="Positive sequence series reactance of the winding.  For a two winding transformer, the full reactance of the transformer should be entered on the primary (high voltage) winding.Positive sequence series reactance of the winding.  For a two winding transformer, the full reactance of the transformer should be entered on the primary (high voltage) winding.")

    # Magnetizing branch conductance (G mag).Magnetizing branch conductance (G mag).
    g = Conductance(desc="Magnetizing branch conductance (G mag).Magnetizing branch conductance (G mag).")

    # Zero sequence series resistance of the winding.Zero sequence series resistance of the winding.
    r0 = Resistance(desc="Zero sequence series resistance of the winding.Zero sequence series resistance of the winding.")

    # DC resistance of the winding.DC resistance of the winding.
    r = Resistance(desc="DC resistance of the winding.DC resistance of the winding.")

    # Magnetizing branch susceptance (B mag).  The value can be positive or negative.Magnetizing branch susceptance (B mag).  The value can be positive or negative.
    b = Susceptance(desc="Magnetizing branch susceptance (B mag).  The value can be positive or negative.Magnetizing branch susceptance (B mag).  The value can be positive or negative.")

    # Zero sequence magnetizing branch conductance.Zero sequence magnetizing branch conductance.
    g0 = Conductance(desc="Zero sequence magnetizing branch conductance.Zero sequence magnetizing branch conductance.")

    # Zero sequence series reactance of the winding.Zero sequence series reactance of the winding.
    x0 = Reactance(desc="Zero sequence series reactance of the winding.Zero sequence series reactance of the winding.")

    # Zero sequence magnetizing branch susceptance.Zero sequence magnetizing branch susceptance.
    b0 = Susceptance(desc="Zero sequence magnetizing branch susceptance.Zero sequence magnetizing branch susceptance.")

    #--------------------------------------------------------------------------
    #  Begin "WindingPiImpedance" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "x", "g", "r0", "r", "b", "g0", "x0", "b0",
                label="Attributes", columns=1),
            VGroup("Model", "Windings",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.WindingPiImpedance",
        title="WindingPiImpedance",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "WindingPiImpedance" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DistributionTapChanger" class:
#------------------------------------------------------------------------------

class DistributionTapChanger(RatioTapChanger):
    """ Additional ratio tap changer parameters common to distribution line regulators. 'tculControlMode' would always be 'volt'. If 'monitoredPhase' is not specified, then if the controlled DistributionTransformerWinding is single-phase, the PT primary is assumed to be connected across that winding, which is the normal case. If the controlled winding is three-phase, then the 'monitoredPhase' is assumed to be 'AN', unless otherwise specified. Whenever 'ctRatio' and 'ptRatio' are specified, it's customary to specify the R and X in 'volts' referred to the PT secondary circuit, otherwise R and X are in feeder primary ohms. If 'ptRatio' is not specified, then 'targetVoltage', 'limitVoltage', and 'bandVoltage' are on the feeder primary base, phase-neutral or phase-phase depending on the 'monitoredPhase'. Otherwise, these attributes are all on the PT secondary base.Additional ratio tap changer parameters common to distribution line regulators. 'tculControlMode' would always be 'volt'. If 'monitoredPhase' is not specified, then if the controlled DistributionTransformerWinding is single-phase, the PT primary is assumed to be connected across that winding, which is the normal case. If the controlled winding is three-phase, then the 'monitoredPhase' is assumed to be 'AN', unless otherwise specified. Whenever 'ctRatio' and 'ptRatio' are specified, it's customary to specify the R and X in 'volts' referred to the PT secondary circuit, otherwise R and X are in feeder primary ohms. If 'ptRatio' is not specified, then 'targetVoltage', 'limitVoltage', and 'bandVoltage' are on the feeder primary base, phase-neutral or phase-phase depending on the 'monitoredPhase'. Otherwise, these attributes are all on the PT secondary base.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Line drop compensator resistance setting for normal (forward) power flow.Line drop compensator resistance setting for normal (forward) power flow.
    lineDropR = Resistance(desc="Line drop compensator resistance setting for normal (forward) power flow.Line drop compensator resistance setting for normal (forward) power flow.")

    # Phase voltage controlling this regulator, measured at regulator location.Phase voltage controlling this regulator, measured at regulator location.
    monitoredPhase = PhaseCode(desc="Phase voltage controlling this regulator, measured at regulator location.Phase voltage controlling this regulator, measured at regulator location.")

    # If true, the line drop compensation is to be applied.If true, the line drop compensation is to be applied.
    lineDropCompensation = Bool(desc="If true, the line drop compensation is to be applied.If true, the line drop compensation is to be applied.")

    # Built-in voltage transducer ratio.Built-in voltage transducer ratio.
    ptRatio = Float(desc="Built-in voltage transducer ratio.Built-in voltage transducer ratio.")

    # Built-in current transducer ratio.Built-in current transducer ratio.
    ctRatio = Float(desc="Built-in current transducer ratio.Built-in current transducer ratio.")

    # Line drop compensator resistance setting for reverse power flow.Line drop compensator resistance setting for reverse power flow.
    reverseLineDropR = Resistance(desc="Line drop compensator resistance setting for reverse power flow.Line drop compensator resistance setting for reverse power flow.")

    # Maximum allowed regulated voltage on the PT secondary base, regardless of line drop compensation. Sometimes referred to as first-house protection.Maximum allowed regulated voltage on the PT secondary base, regardless of line drop compensation. Sometimes referred to as first-house protection.
    limitVoltage = Voltage(desc="Maximum allowed regulated voltage on the PT secondary base, regardless of line drop compensation. Sometimes referred to as first-house protection.Maximum allowed regulated voltage on the PT secondary base, regardless of line drop compensation. Sometimes referred to as first-house protection.")

    # Line drop compensator reactance setting for reverse power flow.Line drop compensator reactance setting for reverse power flow.
    reverseLineDropX = Reactance(desc="Line drop compensator reactance setting for reverse power flow.Line drop compensator reactance setting for reverse power flow.")

    # Voltage range (max - min) on the PT secondary base, centered on 'targetVoltage'.Voltage range (max - min) on the PT secondary base, centered on 'targetVoltage'.
    bandVoltage = Voltage(desc="Voltage range (max - min) on the PT secondary base, centered on 'targetVoltage'.Voltage range (max - min) on the PT secondary base, centered on 'targetVoltage'.")

    # Target voltage on the PT secondary base.Target voltage on the PT secondary base.
    targetVoltage = Voltage(desc="Target voltage on the PT secondary base.Target voltage on the PT secondary base.")

    # Line drop compensator reactance setting for normal (forward) power flow.Line drop compensator reactance setting for normal (forward) power flow.
    lineDropX = Reactance(desc="Line drop compensator reactance setting for normal (forward) power flow.Line drop compensator reactance setting for normal (forward) power flow.")

    #--------------------------------------------------------------------------
    #  Begin "DistributionTapChanger" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "stepVoltageIncrement", "subsequentDelay", "neutralStep", "normalStep", "ltcFlag", "neutralU", "lowStep", "initialDelay", "regulationStatus", "highStep", "tculControlMode", "lineDropR", "monitoredPhase", "lineDropCompensation", "ptRatio", "ctRatio", "reverseLineDropR", "limitVoltage", "reverseLineDropX", "bandVoltage", "targetVoltage", "lineDropX",
                label="Attributes", columns=2),
            VGroup("Model", "GeoLocation", "PSRType", "SvTapStep", "Winding",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.DistributionTapChanger",
        title="DistributionTapChanger",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DistributionTapChanger" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "PerLengthSequenceImpedance" class:
#------------------------------------------------------------------------------

class PerLengthSequenceImpedance(IdentifiedObject):
    """ Sequence impedance and admittance parameters per unit length, for transposed lines of 1, 2, or 3 phases. For 1-phase lines, define x=x0=xself. For 2-phase lines, define x=xs-xm and x0=xs+xm.Sequence impedance and admittance parameters per unit length, for transposed lines of 1, 2, or 3 phases. For 1-phase lines, define x=x0=xself. For 2-phase lines, define x=xs-xm and x0=xs+xm.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All conductor segments described by this sequence impedance.All conductor segments described by this sequence impedance.
    ConductorSegments = List(Instance("CDPSM.IEC61968.WiresExt.DistributionLineSegment"),
        desc="All conductor segments described by this sequence impedance.All conductor segments described by this sequence impedance.")

    # Positive sequence series reactance, per unit of length.Positive sequence series reactance, per unit of length.
    x = Reactance(desc="Positive sequence series reactance, per unit of length.Positive sequence series reactance, per unit of length.")

    # Positive sequence shunt (charging) conductance, per unit of length.Positive sequence shunt (charging) conductance, per unit of length.
    gch = Conductance(desc="Positive sequence shunt (charging) conductance, per unit of length.Positive sequence shunt (charging) conductance, per unit of length.")

    # Zero sequence series resistance, per unit of length.Zero sequence series resistance, per unit of length.
    r0 = Resistance(desc="Zero sequence series resistance, per unit of length.Zero sequence series resistance, per unit of length.")

    # Zero sequence series reactance, per unit of length.Zero sequence series reactance, per unit of length.
    x0 = Reactance(desc="Zero sequence series reactance, per unit of length.Zero sequence series reactance, per unit of length.")

    # Zero sequence shunt (charging) conductance, per unit of length.Zero sequence shunt (charging) conductance, per unit of length.
    g0ch = Conductance(desc="Zero sequence shunt (charging) conductance, per unit of length.Zero sequence shunt (charging) conductance, per unit of length.")

    # Zero sequence shunt (charging) susceptance, per unit of length.Zero sequence shunt (charging) susceptance, per unit of length.
    b0ch = Susceptance(desc="Zero sequence shunt (charging) susceptance, per unit of length.Zero sequence shunt (charging) susceptance, per unit of length.")

    # Positive sequence series resistance, per unit of length.Positive sequence series resistance, per unit of length.
    r = Resistance(desc="Positive sequence series resistance, per unit of length.Positive sequence series resistance, per unit of length.")

    # Positive sequence shunt (charging) susceptance, per unit of length.Positive sequence shunt (charging) susceptance, per unit of length.
    bch = Susceptance(desc="Positive sequence shunt (charging) susceptance, per unit of length.Positive sequence shunt (charging) susceptance, per unit of length.")

    #--------------------------------------------------------------------------
    #  Begin "PerLengthSequenceImpedance" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "x", "gch", "r0", "x0", "g0ch", "b0ch", "r", "bch",
                label="Attributes", columns=1),
            VGroup("Model", "ConductorSegments",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.PerLengthSequenceImpedance",
        title="PerLengthSequenceImpedance",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "PerLengthSequenceImpedance" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TransformerBank" class:
#------------------------------------------------------------------------------

class TransformerBank(Equipment):
    """ An assembly of transformers that are connected together. For three-phase transformers, there would be one transformer per bank. For banks of single-phase transformers, there will be more than one transformer per bank, and they need not be identical.An assembly of transformers that are connected together. For three-phase transformers, there would be one transformer per bank. For banks of single-phase transformers, there will be more than one transformer per bank, and they need not be identical.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All transformers that belong to this bank.All transformers that belong to this bank.
    Transformers = List(Instance("CDPSM.IEC61968.WiresExt.DistributionTransformer"),
        desc="All transformers that belong to this bank.All transformers that belong to this bank.")

    # Vector group of the bank for protective relaying, e.g., Dyn1. For unbalanced transformers, this may not be simply determined from the constituent winding connections.Vector group of the bank for protective relaying, e.g., Dyn1. For unbalanced transformers, this may not be simply determined from the constituent winding connections.
    vectorGroup = Str(desc="Vector group of the bank for protective relaying, e.g., Dyn1. For unbalanced transformers, this may not be simply determined from the constituent winding connections.Vector group of the bank for protective relaying, e.g., Dyn1. For unbalanced transformers, this may not be simply determined from the constituent winding connections.")

    #--------------------------------------------------------------------------
    #  Begin "TransformerBank" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "normaIlyInService", "vectorGroup",
                label="Attributes"),
            VGroup("Model", "GeoLocation", "PSRType", "EquipmentContainer", "Transformers",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.TransformerBank",
        title="TransformerBank",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TransformerBank" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "PerLengthPhaseImpedance" class:
#------------------------------------------------------------------------------

class PerLengthPhaseImpedance(IdentifiedObject):
    """ Impedance and admittance parameters per unit length for n-wire unbalanced lines, in matrix form.Impedance and admittance parameters per unit length for n-wire unbalanced lines, in matrix form.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # All data that belong to this conductor phase impedance.All data that belong to this conductor phase impedance.
    PhaseImpedanceData = List(Instance("CDPSM.IEC61968.WiresExt.PhaseImpedanceData"),
        desc="All data that belong to this conductor phase impedance.All data that belong to this conductor phase impedance.")

    # All conductor segments described by this phase impedance.All conductor segments described by this phase impedance.
    ConductorSegments = List(Instance("CDPSM.IEC61968.WiresExt.DistributionLineSegment"),
        desc="All conductor segments described by this phase impedance.All conductor segments described by this phase impedance.")

    # Number of phase, neutral, and other wires retained. Constrains the number of matrix elements and the phase codes that can be used with this matrix.Number of phase, neutral, and other wires retained. Constrains the number of matrix elements and the phase codes that can be used with this matrix.
    conductorCount = Int(desc="Number of phase, neutral, and other wires retained. Constrains the number of matrix elements and the phase codes that can be used with this matrix.Number of phase, neutral, and other wires retained. Constrains the number of matrix elements and the phase codes that can be used with this matrix.")

    #--------------------------------------------------------------------------
    #  Begin "PerLengthPhaseImpedance" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "conductorCount",
                label="Attributes"),
            VGroup("Model", "PhaseImpedanceData", "ConductorSegments",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.PerLengthPhaseImpedance",
        title="PerLengthPhaseImpedance",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "PerLengthPhaseImpedance" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DistributionTransformer" class:
#------------------------------------------------------------------------------

class DistributionTransformer(Equipment):
    """ An assembly of two or more coupled windings that transform electrical power between voltage levels. Supports both balanced and unbalanced winding connections. This class differs from Wires::PowerTransformer as follows: - it is part of a TransformerBank - it draws parameters exclusively from TransformerInfo and its associated classes.An assembly of two or more coupled windings that transform electrical power between voltage levels. Supports both balanced and unbalanced winding connections. This class differs from Wires::PowerTransformer as follows: - it is part of a TransformerBank - it draws parameters exclusively from TransformerInfo and its associated classes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Transformer data.Transformer data.
    TransformerInfo = Instance("CDPSM.IEC61968.AssetModels.TransformerInfo",
        desc="Transformer data.Transformer data.",
        transient=True,
        opposite="Transformers",
        editor=InstanceEditor(name="_transformerinfos"))

    def _get_transformerinfos(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.AssetModels.TransformerInfo" ]
        else:
            return []

    _transformerinfos = Property(fget=_get_transformerinfos)

    # All windings of this transformer.All windings of this transformer.
    Windings = List(Instance("CDPSM.IEC61968.WiresExt.DistributionTransformerWinding"),
        desc="All windings of this transformer.All windings of this transformer.")

    # Bank this transformer belongs to.Bank this transformer belongs to.
    TransformerBank = Instance("CDPSM.IEC61968.WiresExt.TransformerBank",
        desc="Bank this transformer belongs to.Bank this transformer belongs to.",
        transient=True,
        opposite="Transformers",
        editor=InstanceEditor(name="_transformerbanks"))

    def _get_transformerbanks(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.WiresExt.TransformerBank" ]
        else:
            return []

    _transformerbanks = Property(fget=_get_transformerbanks)

    #--------------------------------------------------------------------------
    #  Begin "DistributionTransformer" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "mRID", "description", "name", "localName", "aliasName", "normaIlyInService",
                label="Attributes"),
            VGroup("Model", "GeoLocation", "PSRType", "EquipmentContainer", "TransformerInfo", "Windings", "TransformerBank",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.DistributionTransformer",
        title="DistributionTransformer",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DistributionTransformer" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "PhaseImpedanceData" class:
#------------------------------------------------------------------------------

class PhaseImpedanceData(Element):
    """ Triplet of resistance, reactance, and susceptance matrix element values.Triplet of resistance, reactance, and susceptance matrix element values.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Conductor phase impedance to which this data belongs.Conductor phase impedance to which this data belongs.
    PhaseImpedance = Instance("CDPSM.IEC61968.WiresExt.PerLengthPhaseImpedance", allow_none=False,
        desc="Conductor phase impedance to which this data belongs.Conductor phase impedance to which this data belongs.",
        transient=True,
        opposite="PhaseImpedanceData",
        editor=InstanceEditor(name="_perlengthphaseimpedances"))

    def _get_perlengthphaseimpedances(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CDPSM.IEC61968.WiresExt.PerLengthPhaseImpedance" ]
        else:
            return []

    _perlengthphaseimpedances = Property(fget=_get_perlengthphaseimpedances)

    # Reactance matrix element value, per length of unit.Reactance matrix element value, per length of unit.
    x = Reactance(desc="Reactance matrix element value, per length of unit.Reactance matrix element value, per length of unit.")

    # Column-wise element index, assuming a symmetrical matrix. Ranges from 1 to N + N*(N-1)/2.Column-wise element index, assuming a symmetrical matrix. Ranges from 1 to N + N*(N-1)/2.
    sequenceNumber = Int(desc="Column-wise element index, assuming a symmetrical matrix. Ranges from 1 to N + N*(N-1)/2.Column-wise element index, assuming a symmetrical matrix. Ranges from 1 to N + N*(N-1)/2.")

    # Susceptance matrix element value, per length of unit.Susceptance matrix element value, per length of unit.
    b = Susceptance(desc="Susceptance matrix element value, per length of unit.Susceptance matrix element value, per length of unit.")

    # Resistance matrix element value, per length of unit.Resistance matrix element value, per length of unit.
    r = Resistance(desc="Resistance matrix element value, per length of unit.Resistance matrix element value, per length of unit.")

    #--------------------------------------------------------------------------
    #  Begin "PhaseImpedanceData" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "x", "sequenceNumber", "b", "r",
                label="Attributes"),
            VGroup("Model", "PhaseImpedance",
                label="References"),
            dock="tab"),
        id="CDPSM.IEC61968.WiresExt.PhaseImpedanceData",
        title="PhaseImpedanceData",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "PhaseImpedanceData" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" The domain package is a data dictionary of quantities and units that define datatypes for attributes (properties) that may be used by any class in any other package.  This package contains the definition of primitive datatypes, including units of measure and permissible values. Each datatype contains a value attribute and an optional unit of measure, which is specified as a static variable initialized to the textual description of the unit of measure. The value of the 'units' string may be country or customer specific. Typical values are given. Permissible values for enumerations are listed in the documentation for the attribute using UML constraint syntax inside curly braces. Lengths of variable strings are listed in the descriptive text where required.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------




from enthought.traits.api import Float, Str, Int, Enum
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------
AngleDegrees = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatMeasurement of angle in degrees")
Reactance = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatReactance (imaginary part of impedance), at rated frequency.")
ActivePower = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatProduct of RMS value of the voltage and the RMS value of the in-phase component of the current")
Resistance = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatResistance (real part of impedance).")
CostPerEnergyUnit = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatCost, in units of currency, per quantity of electrical energy generated.")
AbsoluteDate = Str(desc="http://www.w3.org/2001/XMLSchema#stringDate and time as 'yyyy-mm-dd', which conforms with ISO 8601. UTC time zone is specified as 'yyyy-mm-dd'.")
PU = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatPer Unit - a positive or negative value referred to a defined base. Values typically range from -10 to +10.")
Frequency = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatCycles per second")
Seconds = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatTime, in seconds")
Money = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatAmount of money")
Capacitance = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatA Farad, the typical unit, is the capacitance in which a charge of 1 coulomb produces 1 volt potential difference between its terminals.")
PerCent = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatNormally 0 - 100 on a defined base")
CurrentFlow = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatElectrical current (positive flow is out of the ConductingEquipment into the ConnectivityNode)")
Length = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatUnit of length.")
KWActivePower = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatActive power in kilowatts.")
Hours = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatTime, in hours")
CostRate = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatCost, in units of currency, per hour of elapsed time")
Voltage = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatElectrical voltage.")
Susceptance = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatImaginary part of admittance.")
ApparentPower = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatProduct of the RMS value of the voltage and the RMS value of the current")
Temperature = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatValue of temperature in degrees Celsius.")
StringQuantity = Str(desc="http://www.w3.org/2001/XMLSchema#stringQuantity with string value (when it is not important whether it is an integral or a floating point number) and associated unit information.")
RotationSpeed = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatNumber of revolutions per second.")
RealEnergy = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatReal electrical energy")
IntegerQuantity = Int(0, desc="http://www.w3.org/2001/XMLSchema#integerQuantity with integer value and associated unit information.")
Conductance = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatFactor by which voltage must be multiplied to give corresponding power lost from a circuit. Real part of admittance.")
Minutes = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatTime in minutes")
Weight = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatThe weight of an object.")
Pressure = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatPressure in Pascal.")
ActivePowerChangeRate = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#float")
Inductance = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatInductance.")
Damping = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatPer-unit active power variation with per-unit frequency referenced on the system apparent power base. Typical values in range 1.0 - 2.0.")
ReactivePower = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatProduct of RMS value of the voltage and the RMS value of the quadrature component of the current.")
FloatQuantity = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatQuantity with float value and associated unit information.")
Volume = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatVolume.")
AngleRadians = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatPhase angle in radians")
VoltagePerReactivePower = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatVoltage variation with reactive power")
Impedance = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatRatio of voltage to current.")
WaterLevel = Float(0.0, desc="http://www.w3.org/2001/XMLSchema#floatReservoir water level referred to a given datum such as mean sea level.")

# The unit multipliers defined for the CIM
UnitMultiplier = Enum("m", "T", "p", "k", "M", "micro", "n", "d", "G", "c", "none", desc="The unit multipliers defined for the CIM")
# The units defiend for usage in the CIM
UnitSymbol = Enum("m2", "VAr", "m3", "g", "VArh", "F", "Hz", "deg", "W/s", "V", "V/VAr", "rad", "min", "ohm", "m", "H", "s", "W/Hz", "kg/J", "Wh", "VA", "S", "none", "degC", "s-1", "J", "N", "h", "J/s", "Hz-1", "Pa", "W", "A", "VAh", desc="The units defiend for usage in the CIM")
# Monetary amount per energy unit.
MonetaryAmountPerEnergyUnit = Enum("USD_per_Wh", "EUR_per_Wh", desc="Monetary amount per energy unit.")
# Monetary amount per second.
MonetaryAmountRate = Enum("EUR_per_s", "USD_per_s", desc="Monetary amount per second.")
# Monetary amount per joule.
MonetaryAmountPerHeatUnit = Enum("EUR_per_J", "USD_per_J", desc="Monetary amount per joule.")
# Monetary currencies. Apologies for this list not being exhaustive.
Currency = Enum("EUR", "other", "JPY", "DKK", "NOK", "CNY", "USD", "INR", "SEK", "AUD", "CHF", "CAD", "RUR", "GBP", desc="Monetary currencies. Apologies for this list not being exhaustive.")



# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


""" Defines a map of class names to their module.
"""

dynamics_pkg_map = {
    "Element": "dynamics",
    "Model": "dynamics",
    "RotatingMachine": "dynamics.dynamics",
    "MetaBlockConOutput": "dynamics.dynamics",
    "AttributeBlockParameter": "dynamics.dynamics",
    "MetaBlock": "dynamics.dynamics",
    "BlockParameter": "dynamics.dynamics",
    "MetaBlockReference": "dynamics.dynamics",
    "ExcitationSystemLimiter": "dynamics.dynamics",
    "MetaBlockConnectivity": "dynamics.dynamics",
    "ProtectiveDevice": "dynamics.dynamics",
    "MetaBlockInput": "dynamics.dynamics",
    "SourceModels": "dynamics.dynamics",
    "MetaBlockOutputReference": "dynamics.dynamics",
    "MetaBlockOutput": "dynamics.dynamics",
    "MetaBlockParameter": "dynamics.dynamics",
    "BlockConnection": "dynamics.dynamics",
    "MetaBlockConInput": "dynamics.dynamics",
    "StaticVarDevice": "dynamics.dynamics",
    "MetaBlockConnection": "dynamics.dynamics",
    "BlockConnectivity": "dynamics.dynamics",
    "MetaBlockConSignal": "dynamics.dynamics",
    "Block": "dynamics.dynamics",
    "MetaBlockStateReference": "dynamics.dynamics",
    "AsynchronousMachine": "dynamics.dynamics",
    "TieToMeasurement": "dynamics.dynamics",
    "MetaBlockSignal": "dynamics.dynamics",
    "MetaBlockInputReference": "dynamics.dynamics",
    "MetaBlockParameterReference": "dynamics.dynamics",
    "MetaBlockConnectable": "dynamics.dynamics",
    "MetaBlockState": "dynamics.dynamics",
    "LoadStaticSystem": "dynamics.dynamics.loads",
    "LoadStaticOwner": "dynamics.dynamics.loads",
    "LoadStatic": "dynamics.dynamics.loads",
    "AggregateLoad": "dynamics.dynamics.loads",
    "LoadMotor": "dynamics.dynamics.loads",
    "LoadStaticBus": "dynamics.dynamics.loads",
    "LoadStaticZone": "dynamics.dynamics.loads",
    "LoadStaticArea": "dynamics.dynamics.loads",
    "ExcBAS": "dynamics.dynamics.excitation_systems",
    "ExcDC2A": "dynamics.dynamics.excitation_systems",
    "ExcSEXS": "dynamics.dynamics.excitation_systems",
    "ExcELIN2": "dynamics.dynamics.excitation_systems",
    "ExcAC4A": "dynamics.dynamics.excitation_systems",
    "ExcSK": "dynamics.dynamics.excitation_systems",
    "ExcAC2A": "dynamics.dynamics.excitation_systems",
    "ExcELIN1": "dynamics.dynamics.excitation_systems",
    "ExcST6B": "dynamics.dynamics.excitation_systems",
    "ExcST4B": "dynamics.dynamics.excitation_systems",
    "ExcWT3E": "dynamics.dynamics.excitation_systems",
    "ExcPIC": "dynamics.dynamics.excitation_systems",
    "ExcSK2": "dynamics.dynamics.excitation_systems",
    "ExcST2A": "dynamics.dynamics.excitation_systems",
    "ExcHU": "dynamics.dynamics.excitation_systems",
    "ExcREXS": "dynamics.dynamics.excitation_systems",
    "ExcST7B": "dynamics.dynamics.excitation_systems",
    "ExcAC1A": "dynamics.dynamics.excitation_systems",
    "ExcDC4B": "dynamics.dynamics.excitation_systems",
    "ExcDC1A": "dynamics.dynamics.excitation_systems",
    "ExcAC3A": "dynamics.dynamics.excitation_systems",
    "ExcAC6A": "dynamics.dynamics.excitation_systems",
    "ExcAC5A": "dynamics.dynamics.excitation_systems",
    "ExcST5B": "dynamics.dynamics.excitation_systems",
    "ExcSCRX": "dynamics.dynamics.excitation_systems",
    "ExcAC8B": "dynamics.dynamics.excitation_systems",
    "ExcWT2E": "dynamics.dynamics.excitation_systems",
    "ExcAC7B": "dynamics.dynamics.excitation_systems",
    "ExcWT4E": "dynamics.dynamics.excitation_systems",
    "ExcST1A": "dynamics.dynamics.excitation_systems",
    "ExcBBC": "dynamics.dynamics.excitation_systems",
    "ExcitationSystem": "dynamics.dynamics.excitation_systems",
    "ExcDC3A": "dynamics.dynamics.excitation_systems",
    "ExcST3A": "dynamics.dynamics.excitation_systems",
    "ExcCZ": "dynamics.dynamics.excitation_systems",
    "GovHydro3": "dynamics.dynamics.turbine_governors",
    "GovHydro2": "dynamics.dynamics.turbine_governors",
    "GovGAST": "dynamics.dynamics.turbine_governors",
    "GovWT1T": "dynamics.dynamics.turbine_governors",
    "TurbineGovernor": "dynamics.dynamics.turbine_governors",
    "GovWT2P": "dynamics.dynamics.turbine_governors",
    "GovHydroDD": "dynamics.dynamics.turbine_governors",
    "GovHydroWEH": "dynamics.dynamics.turbine_governors",
    "GovWT3T": "dynamics.dynamics.turbine_governors",
    "GovHydroPID": "dynamics.dynamics.turbine_governors",
    "GovHydro4": "dynamics.dynamics.turbine_governors",
    "GovDUM": "dynamics.dynamics.turbine_governors",
    "GovHydroWPID": "dynamics.dynamics.turbine_governors",
    "GovWT4P": "dynamics.dynamics.turbine_governors",
    "TLCFB1": "dynamics.dynamics.turbine_governors",
    "GovGASM": "dynamics.dynamics.turbine_governors",
    "GovHydroR": "dynamics.dynamics.turbine_governors",
    "GovSteamFV2": "dynamics.dynamics.turbine_governors",
    "GovRAV": "dynamics.dynamics.turbine_governors",
    "GovCT2": "dynamics.dynamics.turbine_governors",
    "GovSteam1": "dynamics.dynamics.turbine_governors",
    "GovSteamCC": "dynamics.dynamics.turbine_governors",
    "GovSteamSGO": "dynamics.dynamics.turbine_governors",
    "GovSteamFV3": "dynamics.dynamics.turbine_governors",
    "GovWT3P": "dynamics.dynamics.turbine_governors",
    "GovWT1P": "dynamics.dynamics.turbine_governors",
    "GovSteam0": "dynamics.dynamics.turbine_governors",
    "GovCT1": "dynamics.dynamics.turbine_governors",
    "GovWT2T": "dynamics.dynamics.turbine_governors",
    "GovHydro1": "dynamics.dynamics.turbine_governors",
    "GovHydroPID2": "dynamics.dynamics.turbine_governors",
    "GovWT4T": "dynamics.dynamics.turbine_governors",
    "GovSteamEU": "dynamics.dynamics.turbine_governors",
    "GovHydro0": "dynamics.dynamics.turbine_governors",
    "GovGASTWD": "dynamics.dynamics.turbine_governors",
    "GovGAST2": "dynamics.dynamics.turbine_governors",
    "PowerSystemStabilizer": "dynamics.dynamics.power_system_stabilizers",
    "PssIEEE4B": "dynamics.dynamics.power_system_stabilizers",
    "PssIEEE2B": "dynamics.dynamics.power_system_stabilizers",
    "PssSB4": "dynamics.dynamics.power_system_stabilizers",
    "PssSB": "dynamics.dynamics.power_system_stabilizers",
    "PssSK": "dynamics.dynamics.power_system_stabilizers",
    "PssPTIST3": "dynamics.dynamics.power_system_stabilizers",
    "PssIEEE1A": "dynamics.dynamics.power_system_stabilizers",
    "PssIEEE3B": "dynamics.dynamics.power_system_stabilizers",
    "PssWSCC": "dynamics.dynamics.power_system_stabilizers",
    "PssPTIST1": "dynamics.dynamics.power_system_stabilizers",
    "PssSH": "dynamics.dynamics.power_system_stabilizers",
    "GenSync": "dynamics.dynamics.generators",
    "GenLoad": "dynamics.dynamics.generators",
    "GenAsync": "dynamics.dynamics.generators",
    "GenEquiv": "dynamics.dynamics.generators",
    "VcompIEEE": "dynamics.dynamics.voltage_compensator",
    "VoltageCompensator": "dynamics.dynamics.voltage_compensator",
    "VcompCross": "dynamics.dynamics.voltage_compensator",
    "MotorAsync": "dynamics.dynamics.motors",
    "MechanicalLoad": "dynamics.dynamics.motors",
    "MechLoad1": "dynamics.dynamics.motors",
    "SynchronousMotorType": "dynamics.dynamics.motors",
    "MotorSync": "dynamics.dynamics.motors",
}
